"""
This file contains blueprint/solution-related object_action functions for processing Egeria Markdown
"""

import json
import sys
from datetime import datetime
from typing import Optional

from loguru import logger
from pygments.lexers import blueprint
from rich import print
from rich.console import Console
from rich.markdown import Markdown

from md_processing.md_processing_utils.common_md_proc_utils import (parse_upsert_command, parse_view_command)
from md_processing.md_processing_utils.common_md_utils import update_element_dictionary, set_element_prop_body, \
    set_update_body, set_create_body, add_search_keywords, add_note_in_dr_e
from md_processing.md_processing_utils.extraction_utils import (extract_command_plus, update_a_command)
from md_processing.md_processing_utils.md_processing_constants import (load_commands)
from pyegeria import body_slimmer, EgeriaTech, PyegeriaException, print_basic_exception

load_commands('commands.json')

console = Console(width=int(200))


@logger.catch
def sync_chain_related_elements(egeria_client: EgeriaTech, guid:str, in_supply_chain_guids:list, display_name:str,
                               replace_all_props:bool):
    if replace_all_props:
        rel_el_list = egeria_client._get_supply_chain_rel_elements(guid)
        if rel_el_list is None:
            logger.warning("Unexpected -> the list was None - assigning empty list")
            rel_el_list = {}

        as_is_parent_guids = set(rel_el_list.get("parent_guids", []))

        to_be_parent_guids = set(in_supply_chain_guids) if in_supply_chain_guids is not None else set()

        logger.trace(
            f"as_is_parent supply chains: {list(as_is_parent_guids)} to_be_parent supply chains: {list(to_be_parent_guids)}")


        parent_guids_to_remove =  as_is_parent_guids - to_be_parent_guids
        logger.trace(f"parent_guids_to_remove: {list(parent_guids_to_remove)}")
        if len(parent_guids_to_remove) > 0:
            for parent_guid in parent_guids_to_remove:
                egeria_client.decompose_info_supply_chains(parent_guid, guid,  None)
                msg = f"Removed `{display_name}` from supply chain parent `{parent_guid}`"
                logger.trace(msg)

        parent_guids_to_add = to_be_parent_guids - as_is_parent_guids
        logger.trace(f"parent supply chains_to_add: {list(parent_guids_to_add)}")
        if len(parent_guids_to_add) > 0:
            for parent_guid in parent_guids_to_add:
                egeria_client.compose_info_supply_chains(parent_guid, guid, None)
                msg = f"Added `{display_name}` to supply chain parent `{parent_guid}`"
                logger.trace(msg)

    else:  # merge - add supply chain to parents
        if in_supply_chain_guids:
            for parent_guid in in_supply_chain_guids:
                egeria_client.compose_info_supply_chains(parent_guid, guid, None)
                msg = f"Added `{display_name}` to supply chain `{parent_guid}`"
                logger.trace(msg)



@logger.catch
def sync_component_related_elements(egeria_client: EgeriaTech, object_type: str,
                                    supply_chain_guids: list, parent_component_guids: list,
                                    actor_guids: list, in_blueprint_guids: list, keywords: list[str], guid: str, qualified_name: str,
                                    display_name: str, merge_update: bool = True) -> None:
    """Sync a components related elements.

    """
    if not merge_update:
        rel_el_list = egeria_client.get_component_related_elements(guid)
        # should I throw an exception if empty?

        as_is_actors = set(rel_el_list.get("actor_guids", []))
        as_is_blueprints = set(rel_el_list.get("blueprint_guids", []))
        as_is_parent_components = set(rel_el_list.get("parent_component_guids", []))
        as_is_supply_chains = set(rel_el_list.get("supply_chain_guids", []))
        as_is_keywords_list = set(rel_el_list.get("keywords_list", {}))
        as_is_keywords = set(rel_el_list.get("keywords_list", {}).keys())


        to_be_actors = set(actor_guids) if actor_guids is not None else set()
        to_be_blueprints = set(in_blueprint_guids) if in_blueprint_guids is not None else set()
        to_be_parent_components = set(parent_component_guids) if parent_component_guids is not None else set()
        to_be_supply_chains = set(supply_chain_guids) if supply_chain_guids is not None else set()
        to_be_keywords_list = set(keywords) if keywords is not None else set()


        logger.trace(
            f"as_is_sub_components: {list(as_is_parent_components)} to_be_sub_components: {list(to_be_parent_components)}")
        logger.trace(f"as_is_actors: {list(as_is_actors)} to_be_actors: {list(to_be_actors)}")
        logger.trace(f"as_is_blueprints: {list(as_is_blueprints)} to_be_blueprints: {list(to_be_blueprints)}")

        keywords_to_add = to_be_keywords_list - as_is_keywords
        logger.trace(f"keywords_to_add: {list(keywords_to_add)}")
        if len(keywords_to_add) > 0:
            for ds in keywords_to_add:
                egeria_client.add_search_keyword_to_element(guid, ds)
                msg = f"Added `{ds}` to component `{guid}`"
                logger.trace(msg)

        keywords_to_remove = as_is_keywords_list - to_be_keywords_list
        logger.trace(f"keyword_to_remove: {list(keywords_to_remove)}")
        if len(keywords_to_remove) > 0:
            for ds in keywords_to_remove: ## change structure of get related elements to return pairs of keywords and guids
                egeria_client.remove_search_keyword(rel_el_list['keywords_list'][ds])
                msg = f"Removed `{ds}` from component `{guid}`"
                logger.trace(msg)

        parent_components_to_remove = as_is_parent_components - to_be_parent_components
        logger.trace(f"sub_components_to_remove: {list(parent_components_to_remove)}")
        if len(parent_components_to_remove) > 0:
            for ds in parent_components_to_remove:
                egeria_client.detach_sub_component(ds, guid, None)
                msg = f"Removed `{display_name}` from component `{ds}`"
                logger.trace(msg)



        parent_components_to_add = to_be_parent_components - as_is_parent_components
        logger.trace(f"parent_components_to_add: {list(parent_components_to_add)}")
        if len(parent_components_to_add) > 0:
            for ds in parent_components_to_add:
                egeria_client.link_subcomponent(ds, guid, None)
                msg = f"Added `{display_name}` to component `{ds}`"
                logger.trace(msg)

        blueprints_to_remove = as_is_blueprints - to_be_blueprints
        logger.trace(f"blueprints_to_remove: {list(blueprints_to_remove)}")
        if len(blueprints_to_remove) > 0:
            for bp in blueprints_to_remove:
                egeria_client.detach_solution_component_from_blueprint(bp, guid, None)
                msg = f"Removed `{display_name}` from blueprintt `{bp}`"
                logger.trace(msg)

        blueprints_to_add = to_be_blueprints - as_is_blueprints
        logger.trace(f"blueprints_to_add: {list(blueprints_to_add)}")
        if len(blueprints_to_add) > 0:
            for bp in blueprints_to_add:
                egeria_client.link_solution_component_to_blueprint(bp, guid, None)
                msg = f"Added `{display_name}` to component `{bp}`"
                logger.trace(msg)



        actors_to_remove = to_be_actors - as_is_actors
        logger.trace(f"actors_to_remove: {list(actors_to_remove)}")
        if len(actors_to_remove) > 0:
            for actor in actors_to_remove:
                egeria_client.detach_component_actore(actor, guid, None)
                msg = f"Removed actor `{actor}` from component `{display_name}`"
                logger.trace(msg)

        actors_to_add = to_be_actors - as_is_actors
        logger.trace(f"actors_to_add: {list(actors_to_add)}")
        if len(actors_to_add) > 0:
            for actor in actors_to_add:
                egeria_client.link_component_to_actor(actor, guid, None)
                msg = f"Added `{display_name}` to role `{actor}`"
                logger.trace(msg)

        supply_chains_to_remove = as_is_supply_chains - to_be_supply_chains
        logger.trace(f"supply_chains_to_remove: {list(supply_chains_to_remove)}")
        if len(supply_chains_to_remove) > 0:
            for isc in supply_chains_to_remove:
                egeria_client.detach_design_from_implementation(isc, guid)
                msg = f"Removed `{isc}` from `{display_name}`"
                logger.trace(msg)
        supply_chains_to_add = to_be_supply_chains - as_is_supply_chains
        logger.trace(f"supply_chains_to_add: {list(supply_chains_to_add)}")
        if len(supply_chains_to_add) > 0:
            body = {
                "class": "NewRelationshipRequestBody",
                "properties": {
                    "class": "ImplementedByProperties",
                    "description": "a blank description to satisfy the Egeria gods"
                    }
                }
            for isc in supply_chains_to_add:
                egeria_client.link_design_to_implementation(isc, guid, body)
                msg = f"Added `{isc}` to`{display_name}`"
                logger.trace(msg)
            logger.info(f"Replaced the related elements in `{display_name}`")

    else:  # merge - add field to related elements
        if parent_component_guids:
            for comp in parent_component_guids:
                egeria_client.link_subcomponent(guid, comp, None)
                msg = f"Added `{parent_component_guids}` to `{display_name}`"
                logger.trace(msg)

        if actor_guids:
            for actor in actor_guids:
                egeria_client.link_component_to_actor(actor, guid, None)
                msg = f"Added `{actor_guids}` to `{display_name}`"
                logger.trace(msg)

        if in_blueprint_guids:
            for bp in in_blueprint_guids:
                egeria_client.link_solution_component_to_blueprint(bp, guid, None)
                msg = f"Added `{in_blueprint_guids}` to `{display_name}`"
                logger.trace(msg)

        if keywords:
            add_search_keywords(egeria_client, guid, keywords)

        if supply_chain_guids:
            body = {
                "class": "NewRelationshipRequestBody",
                "properties": {
                    "class": "ImplementedByProperties",
                    "description": "a blank description to satisfy the Egeria gods"
                }
            }
            for isc in supply_chain_guids:
                egeria_client.link_design_to_implementation(isc, guid, body)
                msg = f"Added `{display_name}` to `{isc}`"
                logger.trace(msg)

        logger.info(f"Merged related elements in `{display_name}`")


def sync_blueprint_related_elements(egeria_client: EgeriaTech, object_type: str, component_guids: list, guid: str,
                                    qualified_name: str, display_name: str, replace_all_props: bool = True) -> None:
    """Sync a blueprints related elements.

    """
    if replace_all_props:
        bp_element = egeria_client.get_solution_blueprint_by_guid(guid)
        solution_components = bp_element['solutionComponents']
        as_is_components = {}
        if len(solution_components) > 0:
            for component in solution_components:
                as_is_components.append(component['elementHeader']['guid'])

        # should I throw an exception if empty?

        to_be_components = set(component_guids) if component_guids is not None else set()

        logger.trace(f"as_is_components: {list(as_is_components)} to_be_sub_components: {list(to_be_components)}")

        components_to_remove = as_is_components - to_be_components
        logger.trace(f"components_to_remove: {list(components_to_remove)}")
        if len(components_to_remove) > 0:
            for ds in components_to_remove:
                egeria_client.detach_solution_component_from_blueprint(guid, ds, None)
                msg = f"Removed `{ds}` from component `{display_name}`"
                logger.trace(msg)
        components_to_add = to_be_components - as_is_components
        logger.trace(f"sub_components_to_add: {list(components_to_add)}")
        if len(components_to_add) > 0:
            for ds in components_to_add:
                egeria_client.link_solution_component_to_blueprint(guid, ds, None)
                msg = f"Added `{ds}` to component `{display_name}`"
                logger.trace(msg)

        logger.info(f"Replaced the related elements in `{display_name}`")

    else:  # merge - add field to related elements
        if component_guids:
            for comp in component_guids:
                egeria_client.link_solution_component_to_blueprint(guid, comp, None)
            msg = f"Added `{component_guids}` to `{display_name}`"
            logger.trace(msg)

        logger.info(f"Merged related elements in `{display_name}`")


@logger.catch
def process_blueprint_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    """
    Processes a solution blueprint create or update object_action by extracting key attributes such as
    blueprint name, description, and usage from the given text.

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))
    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)
    journal_entry = parsed_output.get('Journey Entry', {}.get('value', None))
    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    description = attributes.get('Description', {}).get('value', None)
    display_name = attributes['Display Name'].get('value', None)
    search_keywords = attributes['Search Keywords'].get('value', None)



    component_guids = attributes.get('Solution Components', {}).get('guid_list', None)

    status = attributes.get('Status', {}).get('value', None)
    if status is None:
        status = "ACTIVE"

    replace_all_props = not attributes.get('Merge Update', {}).get('value', True)

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        prop_body = set_element_prop_body(object_type, qualified_name, attributes)

        try:
            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"object_action\n")
                    logger.error(msg)
                    print(Markdown(msg))
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                    body = set_update_body(object_type, attributes)
                    body['properties'] = prop_body
                    egeria_client.update_solution_blueprint(guid, body)
                    if status:
                        egeria_client.update_solution_element_status(guid, status)

                    msg = f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___"
                    update_element_dictionary(qualified_name, {
                        'guid': guid, 'display_name': display_name
                    })
                    logger.success(msg)
                    sync_blueprint_related_elements(egeria_client, object_type, component_guids, guid, qualified_name,
                                                    display_name, replace_all_props)

                    journal_entry_guid = add_note_in_dr_e(egeria_client, qualified_name, display_name, journal_entry)
                    logger.success(f"===> Updated {object_type} `{display_name}` related elements\n\n")
                    return egeria_client.get_solution_blueprints_by_name(qualified_name, output_format='MD', report_spec = "Solution-Blueprint-DrE")


            elif object_action == "Create":
                print(f"valid: {valid}, type: {type(valid)}")
                try:
                    if valid is False and exists:
                        msg = (f"  Data Specification `{display_name}` already exists and result document updated changing "
                               f"`Create` to `Update` in processed output\n\n___")
                        logger.error(msg)
                        print(Markdown(msg))
                        return update_a_command(txt, object_action, object_type, qualified_name, guid)

                    elif not valid:
                        msg = (f"==>{object_type} `{display_name}` is not valid and can't be created")
                        print(Markdown(msg))
                        logger.error(msg)
                        return

                    else:
                        body = set_create_body(object_type,attributes)
                        body['properties'] = prop_body
                        guid = egeria_client.create_solution_blueprint(body)
                except Exception as e:
                    print(f"Unexpected error: {e}, {type(valid)}, {valid}")


                if guid:
                    update_element_dictionary(qualified_name, {
                        'guid': guid, 'display_name': display_name
                        })
                    journal_entry_guid = add_note_in_dr_e(egeria_client, qualified_name, display_name, journal_entry)
                    msg = f"\n\nCreated Element `{display_name}` with GUID {guid}\n\n___"
                    print(Markdown(msg))
                    logger.success(msg)
                    return egeria_client.get_solution_blueprint_by_guid(guid, output_format='MD', report_spec = "Solution-Blueprint-DrE")
                else:
                    msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                    print(Markdown(msg))
                    logger.error(msg)
                    return None

        except PyegeriaException as e:
            print_basic_exception(e)
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None


@logger.catch
def process_solution_component_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> \
        Optional[str]:
    """
    Processes a solution component create or update object_action by extracting key attributes such as
    component name, description, and parent components from the given text.

    :param txt: A string representing the input cell to be processed for
        extracting component-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    description = attributes.get('Description', {}).get('value', None)
    display_name = attributes['Display Name'].get('value', None)
    version_identifier = attributes.get('Version Identifier', {}).get('value', None)
    solution_component_type = attributes.get('Solution Component Type', {}).get('value', None)
    planned_deployed_impl_type = attributes.get('Planned Deployed Implementation Type', {}).get('value', None)
    initial_status = attributes.get('Status', {}).get('value', None)
    url = attributes.get('URL', {}).get('value', None)
    search_keywords = attributes.get('Search Keywords', {}).get('value', None)
    journal_entry = attributes.get('Journal Entry', {}).get('value', None)
    user_defined_status = attributes.get('User Defined Status', {}).get('value', None)
    if initial_status != "OTHER":
        user_defined_status = None

    # sub_component_names = attributes.get('Solution SubComponents', {}).get('name_list', None)
    # sub_component_guids = attributes.get('Solution SubComponents', {}).get('guid_list', None)
    actor_names = attributes.get('Actors', {}).get('name_list', None)
    actor_guids = attributes.get('Actors', {}).get('guid_list', None)
    in_blueprint_names = attributes.get('In Solution Blueprints', {}).get('name_list', None)
    in_blueprint_guids = attributes.get('In Solution Blueprints', {}).get('guid_list', None)
    in_supply_chain_names = attributes.get('In Information Supply Chains', {}).get('name_list', None)
    in_supply_chain_guids = attributes.get('In Information Supply Chains', {}).get('guid_list', None)
    in_component_names = attributes.get('In Solution Components', {}).get('name_list', None)
    in_component_guids = attributes.get('In Solution Components', {}).get('guid_list', None)
    parent_component_guids = attributes.get('Parent Components', {}).get('guid_list', None)
    parent_component_names = attributes.get('Parent Components', {}).get('name_list', None)

    effective_time = attributes.get('Effective Time', {}).get('value', None)
    effective_from = attributes.get('Effective From', {}).get('value', None)
    effective_to = attributes.get('Effective To', {}).get('value', None)
    external_source_guid = attributes.get('External Source GUID', {}).get('value', None)
    external_source_name = attributes.get('External Source Name', {}).get('value', None)

    anchor_guid = attributes.get('Anchor ID', {}).get('guid', None)
    parent_guid = attributes.get('Parent ID', {}).get('guid', None)
    parent_relationship_type_name = attributes.get('Parent Relationship Type Name', {}).get('value', None)
    parent_relationship_properties = attributes.get('Parent Relationship Properties', {}).get('value', None)
    parent_at_end1 = attributes.get('Parent at End1', {}).get('value', True)

    anchor_scope_guid = attributes.get('Anchor Scope GUID', {}).get('value', None)
    is_own_anchor = attributes.get('Is Own Anchor', {}).get('value', True)
    if parent_guid is None:
        is_own_anchor = True

    additional_prop = attributes.get('Additional Properties', {}).get('value', None)
    additional_properties = json.loads(additional_prop) if additional_prop is not None else None
    extended_prop = attributes.get('Extended Properties', {}).get('value', None)
    extended_properties = json.loads(extended_prop) if extended_prop is not None else None

    merge_update = attributes.get('Merge Update', {}).get('value', True)



    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            if object_action == "Update":
                if not exists:
                    msg = (f"==> Element `{display_name}` does not exist! Updating result document with Create."
                           f"object_action\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    logger.error(f"==> Element `{display_name}` entry is not valid. Please refer to the errors above.")
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                body = body_slimmer({
                    "class": "UpdateElementRequestBody",
                    "externalSourceGUID": external_source_guid,
                    "externalSourceName": external_source_name,
                    "effectiveTime": effective_time,
                    "mergeUpdate": merge_update,
                    "forLineage": False,
                    "forDuplicateProcessing": False,
                    "properties": {
                        "class": "SolutionComponentProperties",
                        "qualifiedName": qualified_name,
                        "displayName": display_name,
                        "description": description,
                        "solutionComponentType": solution_component_type,
                        "plannedDeployedImplementationType": planned_deployed_impl_type,
                        "additionalProperties": additional_properties,
                        "extendedProperties": extended_properties,
                        "effectiveFrom": effective_from,
                        "effectiveTo": effective_to,
                        "URL": url
                        }
                    })

                egeria_client.update_solution_component(guid, body)
                msg = f"\n==>Updated  {object_type} `{display_name}` with GUID {guid}\n"
                logger.success(msg)
                print(Markdown(msg))
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                    })
                # Sync Parent Components and Blueprints
                sync_component_related_elements(egeria_client, object_type ,
                                                in_supply_chain_guids,parent_component_guids,actor_guids,
                                                in_blueprint_guids, search_keywords, guid, qualified_name,
                                                display_name,
                                                merge_update)

                if journal_entry:
                    journal_entry_guid = add_note_in_dr_e(egeria_client, qualified_name, display_name, journal_entry)
                return egeria_client.get_solution_component_by_guid(guid, output_format='MD', report_spec = "Solution-Component-DrE")


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"  {object_type} `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                elif not valid:
                    msg = (f"==>{object_type} `{display_name}` is not valid and can't be created")
                    logger.error(msg)
                    return

                else:
                    body = body_slimmer({
                        "class": "NewElementRequestBody",
                        "anchorGUID": anchor_guid,
                        "isOwnAnchor": is_own_anchor,
                        "parentGUID": parent_guid,
                        "parentRelationshipTypeName": parent_relationship_type_name,
                        "parentRelationshipProperties": parent_relationship_properties,
                        "parentAtEnd1": parent_at_end1,
                        "properties": {
                            "class": "SolutionComponentProperties",
                            "qualifiedName": qualified_name,
                            "displayName": display_name,
                            "description": description,
                            "solutionComponentType": solution_component_type,
                            "versionIdentifier" : version_identifier,
                            "plannedDeployedImplementationType": planned_deployed_impl_type,
                            "userDefinedStatus" : user_defined_status,
                            "additionalProperties": additional_properties,
                            "extendedProperties": extended_properties,
                            "effectiveFrom": effective_from,
                            "effectiveTo": effective_to,
                            "URL": url
                        },
                        "initialStatus": initial_status,
                        "externalSourceGUID": external_source_guid,
                        "externalSourceName": external_source_name,
                        "effectiveTime": effective_time,
                        "forLineage": False,
                        "forDuplicateProcessing": False
                    })

                    guid = egeria_client.create_solution_component(body)
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                            })
                        msg = f"\n\n==> Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        print(Markdown(msg))

                        if search_keywords:
                            add_search_keywords(egeria_client, guid, search_keywords)

                        if in_component_guids:
                            for comp in in_component_guids:
                                egeria_client.link_subcomponent(comp, guid, None)
                            msg = f"Added to parent components `{in_component_names}` "
                            logger.trace(msg)

                        if actor_guids:
                            for actor in actor_guids:
                                egeria_client.link_component_to_actor(actor, guid, None)
                            msg = f"Added `{actor_guids}` to `{display_name}`"
                            logger.trace(msg)

                        if in_blueprint_guids:
                            for bp in in_blueprint_guids:
                                egeria_client.link_solution_component_to_blueprint(bp, guid, None)
                            msg = f"Added  `{display_name}`to blueprints `{in_blueprint_names}`"
                            logger.trace(msg)

                        if in_supply_chain_guids:
                            for isc in in_supply_chain_guids:
                                egeria_client.link_design_to_implementation(isc, guid, None)
                            msg = f"Added  `{display_name}`to supply chain `{in_supply_chain_names}`"
                            logger.trace(msg)
                        journal_entry_guid = add_note_in_dr_e(egeria_client, qualified_name, display_name,
                                                              journal_entry)

                        return egeria_client.get_solution_component_by_guid(guid, output_format='MD')
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None


@logger.catch
def process_component_link_unlink_command(egeria_client: EgeriaTech, txt: str,
                                                         directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to wire solution components.

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    component1 = attributes.get('Component1', {}).get('guid', None)
    component2 = attributes.get('Component2', {}).get('guid', None)
    label = attributes.get('Wire Label', {}).get('value', None)
    description = attributes.get('Description', {}).get('value', None)

    valid = parsed_output['valid']
    exists = component1 is not None and  component2 is not None


    external_source_guid = attributes.get('External Source GUID', {}).get('value', None)
    external_source_name = attributes.get('External Source Name', {}).get('value', None)

    effective_time = attributes.get('Effective Time', {}).get('value', None)
    effective_from = attributes.get('Effective From', {}).get('value', None)
    effective_to = attributes.get('Effective To', {}).get('value', None)

    additional_prop = attributes.get('Additional Properties', {}).get('value', None)
    additional_properties = json.loads(additional_prop) if additional_prop is not None else None
    extended_prop = attributes.get('Extended Properties', {}).get('value', None)
    extended_properties = json.loads(extended_prop) if extended_prop is not None else None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            if object_action == "Unlink":
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                    body = body_slimmer({
                        "class": "MetadataSourceRequestBody",
                        "externalSourceGUID": external_source_guid,
                        "externalSourceName": external_source_name,
                        "effectiveTime": effective_time,
                        "forLineage": False,
                        "forDuplicateProcessing": False
                        })

                egeria_client.detach_solution_linking_wire(component1, component2, body)

                logger.success(f"===> Detached segment with {label} from `{component1}`to {component2}\n")
                out = parsed_output['display'].replace('Unlink', 'Link', 1)

                return (out)


            elif object_action == "Link":
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `{label}` is not valid and can't be created"
                    logger.error(msg)
                    return
                else:
                    body = {
                        "class": "NewRelationshipRequestBody",
                        "effectiveTime": effective_time,
                        "forLineage": False,
                        "forDuplicateProcessing": False,
                        "properties": {
                            "class": "SolutionLinkingWireProperties",
                            "label": label,
                            "description": description,
                            "effectiveFrom": effective_from,
                            "effectiveTo": effective_to
                            }
                        }

                    egeria_client.link_solution_linking_wire(component1, component2, body)
                    msg = f"==>Created {object_type} link named `{label}`\n"
                    logger.success(msg)
                    print(Markdown(msg))
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None



@logger.catch
def process_information_supply_chain_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") \
        -> \
        Optional[str]:
    """
    Processes a solution blueprint create or update object_action by extracting key attributes such as
    blueprint name, description, and usage from the given text.

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    description = attributes.get('Description', {}).get('value', None)
    display_name = attributes['Display Name'].get('value', None)
    version_identifier = attributes.get('Version Identifier', {}).get('value', None)
    journal_entry = attributes.get('Journal Entry', {}).get('Value', None)
    effective_time = attributes.get('Effective Time', {}).get('value', None)
    effective_from = attributes.get('Effective From', {}).get('value', None)
    effective_to = attributes.get('Effective To', {}).get('value', None)
    external_source_guid = attributes.get('External Source GUID', {}).get('value', None)
    external_source_name = attributes.get('External Source Name', {}).get('value', None)

    anchor_guid = attributes.get('Anchor ID', {}).get('guid', None)
    parent_guid = attributes.get('Parent ID', {}).get('guid', None)
    parent_relationship_type_name = attributes.get('Parent Relationship Type Name', {}).get('value', None)
    parent_at_end1 = attributes.get('Parent at End1', {}).get('value', True)

    anchor_scope_guid = attributes.get('Anchor Scope GUID', {}).get('value', None)
    is_own_anchor = attributes.get('Is Own Anchor', {}).get('value', True)
    if parent_guid is None:
        is_own_anchor = True
    nested_supply_chain_guids = attributes.get('Nested Supply Chain', {}).get('guids', None)
    additional_prop = attributes.get('Additional Properties', {}).get('value', None)
    additional_properties = json.loads(additional_prop) if additional_prop is not None else None
    extended_prop = attributes.get('Extended Properties', {}).get('value', None)
    extended_properties = json.loads(extended_prop) if extended_prop is not None else None

    scope = attributes.get('Scope', {}).get('value', None)
    purposes = attributes.get('Purposes', {}).get('value', None)
    in_supply_chain_guids = attributes.get('In Information Supply Chain', {}).get('guid_list', None)

    replace_all_props = not attributes.get('Merge Update', {}).get('value', True)

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"object_action\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                    body = body_slimmer({
                        "class": "UpdateElementRequestBody",
                        "externalSourceGUID": external_source_guid,
                        "externalSourceName": external_source_name,
                        "effectiveTime": effective_time,
                        "forLineage": False,
                        "forDuplicateProcessing": False,
                        "properties": {
                            "class": "InformationSupplyChainProperties", "effectiveFrom": effective_from,
                            "effectiveTo": effective_to, "extendedProperties": extended_properties,
                            "qualifiedName": qualified_name, "additionalProperties": additional_properties,
                            "displayName": display_name, "description": description, "scope": scope,
                            "purposes": purposes, "version": version_identifier
                            }
                        })

                egeria_client.update_info_supply_chain(guid, body, replace_all_props)

                sync_chain_related_elements(egeria_client, guid, in_supply_chain_guids, display_name, replace_all_props)
                logger.success(f"==> Updated  {object_type} `{display_name}` with GUID {guid}\n\n")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                    })
                journal_entry_guid = add_note_in_dr_e(egeria_client, qualified_name, display_name, journal_entry)

                logger.success(f"===> Updated {object_type} `{display_name}` related elements\n\n")
                return egeria_client.get_info_supply_chain_by_guid(guid, output_format='MD')


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (
                        f"-->  Data Specification `{display_name}` already exists and result document updated changing "
                        f"`Create` to `Update` in processed output\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                elif valid is False:
                    msg = f"==>{object_type} `{display_name}` is not valid and can't be created"
                    logger.error(msg)
                    return
                else:
                    body = {
                        "class": "NewElementRequestBody",
                        "externalSourceGUID": external_source_guid,
                        "externalSourceName": external_source_name,
                        "forLineage": False,
                        "forDuplicateProcessing": False,
                        "effectiveTime": effective_time,
                        "anchorGUID": anchor_guid,
                        "isOwnAnchor": is_own_anchor,
                        "anchorScopeGUID": anchor_scope_guid,
                        "parentGUID": parent_guid,
                        "parentRelationshipTypeName": parent_relationship_type_name,
                        "parentAtEnd1": parent_at_end1,
                        "properties": {
                            "class": "InformationSupplyChainProperties",
                            "effectiveFrom": effective_from,
                            "effectiveTo": effective_to,
                            "extendedProperties": extended_properties,
                            "qualifiedName": qualified_name,
                            "additionalProperties": additional_properties,
                            "displayName": display_name,
                            "description": description,
                            "scope": scope,
                            "purposes": purposes,
                            "version": version_identifier
                            }
                        }

                    guid = egeria_client.create_info_supply_chain(body)
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                            })
                        if len(in_supply_chain_guids) > 0:
                            for nested_chain in in_supply_chain_guids:
                                egeria_client.compose_info_supply_chains(guid, nested_chain)
                        journal_entry_guid = add_note_in_dr_e(egeria_client, qualified_name, display_name,
                                                              journal_entry)

                        msg = f"==>Created Element `{display_name}` with GUID {guid}\n"
                        logger.success(msg)
                        return egeria_client.get_info_supply_chain_by_guid(guid, output_format='MD')
                    else:
                        msg = f"==>Failed to create element `{display_name}` with GUID {guid}\n"
                        logger.error(msg)
                        return None

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None



@logger.catch
def process_information_supply_chain_link_unlink_command(egeria_client: EgeriaTech, txt: str,
                                                         directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to associate or break up peer supply chains..

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    component1 = attributes.get('component1', {}).get('guid', None)
    component2 = attributes.get('component2', {}).get('guid', None)
    label = attributes.get('Link Label', {}).get('value', None)
    description = attributes.get('Description', {}).get('value', None)

    valid = parsed_output['valid']
    exists = component1 is not None and  component2 is not None


    external_source_guid = attributes.get('External Source GUID', {}).get('value', None)
    external_source_name = attributes.get('External Source Name', {}).get('value', None)

    effective_time = attributes.get('Effective Time', {}).get('value', None)
    effective_from = attributes.get('Effective From', {}).get('value', None)
    effective_to = attributes.get('Effective To', {}).get('value', None)

    additional_prop = attributes.get('Additional Properties', {}).get('value', None)
    additional_properties = json.loads(additional_prop) if additional_prop is not None else None
    extended_prop = attributes.get('Extended Properties', {}).get('value', None)
    extended_properties = json.loads(extended_prop) if extended_prop is not None else None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            if object_action == "Unlink":
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                    body = body_slimmer({
                        "class": "MetadataSourceRequestBody",
                        "externalSourceGUID": external_source_guid,
                        "externalSourceName": external_source_name,
                        "effectiveTime": effective_time,
                        "forLineage": False,
                        "forDuplicateProcessing": False
                        })

                egeria_client.unlink_peer_info_supply_chains(component1, component2, body)

                logger.success(f"===> Detached segment with {label} from `{component1}`to {component2}\n")
                out = parsed_output['display'].replace('Unlink', 'Link', 1)

                return (out)


            elif object_action == "Link":
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `{label}` is not valid and can't be created"
                    logger.error(msg)
                    return
                else:
                    body = {
                        "class": "NewRelationshipRequestBody",
                        "effectiveTime": effective_time,
                        "forLineage": False,
                        "forDuplicateProcessing": False,
                        "properties": {
                            "class": "InformationSupplyChainLinkProperties",
                            "label": label,
                            "description": description,
                            "effectiveFrom": effective_from,
                            "effectiveTo": effective_to
                            }
                        }

                    egeria_client.link_peer_info_supply_chain(component1, component2, body)
                    msg = f"==>Created {object_type} link named `{label}`\n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out


        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None

@logger.catch
def process_sol_arch_list_command(egeria_client: EgeriaTech, txt: str, kind:str, directive: str = "display" ) -> Optional[str]:
    """
    Processes Solution Blueprint list object_action by extracting key attributes such as
     search string from the given text.

    :param txt: A string representing the input cell to be processed for
        extracting term-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    attributes = parsed_output['attributes']

    valid = parsed_output['valid']

    print(Markdown(parsed_output['display']))

    if directive == "display":
        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
            logger.error(msg)
        return valid

    elif directive == "process":
        attributes = parsed_output['attributes']
        search_string = attributes.get('Search String', {}).get('value', '*')
        output_format = attributes.get('Output Format', {}).get('value', 'LIST')
        detailed = attributes.get('Detailed', {}).get('value', False)

        match kind:
            case "Solution Blueprints":
                find_proc = egeria_client.find_solution_blueprints
            case "Information Supply Chains":
                find_proc = egeria_client.find_information_supply_chains
            case "Solution Components":
                find_proc = egeria_client.find_solution_components
            case "Solution Roles":
                find_proc = egeria_client.find_solution_roles

        try:
            if not valid:  # First validate the command before we process it
                msg = f"Validation failed for {object_action} `{object_type}`\n"
                logger.error(msg)
                return None

            list_md = f"\n# {kind} with filter: `{search_string}`\n\n"
            if output_format == "DICT":
                struct = find_proc(search_string, output_format=output_format)
                list_md += f"```{json.dumps(struct, indent=4)}```\n"
            else:
                list_md += find_proc(search_string, output_format=output_format)
            logger.info(f"Wrote Output for search string: `{search_string}`")

            return list_md

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            console.print_exception(show_locals=True)
            return None
    else:
        return None





"""
This file contains product manager commands for processing Egeria Markdown
"""
import json
import os
from typing import Optional

from loguru import logger
from rich import print
from rich.console import Console
from rich.markdown import Markdown

from md_processing.md_processing_utils.common_md_proc_utils import (parse_upsert_command, parse_view_command)
from md_processing.md_processing_utils.common_md_utils import update_element_dictionary, set_update_body, \
    set_element_prop_body, set_delete_request_body, set_rel_request_body, set_create_body, set_object_classifications, \
    set_product_body
from md_processing.md_processing_utils.extraction_utils import (extract_command_plus, update_a_command)
from md_processing.md_processing_utils.md_processing_constants import (load_commands)
from pyegeria import DEBUG_LEVEL, body_slimmer, PyegeriaException, print_basic_exception, print_exception_table
from pyegeria.egeria_tech_client import EgeriaTech
from pyegeria.utils import make_format_set_name_from_type

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get("EGERIA_VIEW_SERVER_URL", "https://localhost:9443")
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get("EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443")
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_WIDTH = os.environ.get("EGERIA_WIDTH", 220)
EGERIA_JUPYTER = os.environ.get("EGERIA_JUPYTER", False)
EGERIA_HOME_GLOSSARY_GUID = os.environ.get("EGERIA_HOME_GLOSSARY_GUID", None)
EGERIA_GLOSSARY_PATH = os.environ.get("EGERIA_GLOSSARY_PATH", None)
EGERIA_ROOT_PATH = os.environ.get("EGERIA_ROOT_PATH", "../../")
EGERIA_INBOX_PATH = os.environ.get("EGERIA_INBOX_PATH", "md_processing/dr_egeria_inbox")
EGERIA_OUTBOX_PATH = os.environ.get("EGERIA_OUTBOX_PATH", "md_processing/dr_egeria_outbox")
LOCAL_QUALIFIER = os.environ.get("EGERIA_LOCAL_QUALIFIER", None)

load_commands('commands.json')
debug_level = DEBUG_LEVEL

console = Console(width=int(200))


#
# Helper functions for the data designer commands
#
@logger.catch
def add_member_to_collections(egeria_client: EgeriaTech, collection_list: list, display_name: str,
                              guid: str) -> None:
    """
    Add member to data dictionaries and data specifications.
    """
    body = {
        "class": "NewRelationshipRequestBody", "properties": {
            "class": "CollectionMembershipProperties", "membershipRationale": "User Specified",
            "notes": "Added by Dr.Egeria"
        }
    }
    try:
        if collection_list is not None:
            for collection in collection_list:
                egeria_client.add_to_collection(collection, guid, body)
                msg = f"Added `{display_name}` member to `{collection}`"
                logger.info(msg)
        else:
            logger.info("There were no data collections to add.")
        return

    except Exception as e:
        console.print_exception()


@logger.catch
def remove_member_from_collections(egeria_client: EgeriaTech, collection_list: list, display_name: str,
                                   guid: str) -> None:
    try:
        for collection in collection_list:
            egeria_client.remove_from_collection(collection, guid)
            msg = f"Removed `{display_name}` member from `{collection}`"
            logger.info(msg)
        return

    except Exception as e:
        console.print_exception()


@logger.catch
def update_data_collection_memberships(egeria_client: EgeriaTech, entity_type: str, guid_list: list,
                                       collection_class: str, guid: str, display_name: str,
                                       replace_all_props: bool = True) -> None:
    """ update the collection membership of the element

        If replace_all_props is set to True, all existing memberships are removed and new memberships are added.
        If replace_all_props is set to False, only the new memberships are added.
    """

    if replace_all_props:
        match entity_type:
            case "Data Specification":
                get_command = egeria_client.get_collection_by_guid
            case "Data Structure":
                get_command = egeria_client.get_data_structure_by_guid
            case "Data Field":
                get_command = egeria_client.get_data_field_by_guid
            case "Data Class":
                get_command = egeria_client.get_data_class_by_guid

        coll_list = egeria_client.get_data_memberships(get_command, guid)
        if coll_list is None:
            logger.warning("Unexpected -> the collection list was None - assigning empty dict")
            coll_list = {}
        # compare the existing collections to desired collections
        if collection_class == "DataDictionary":
            as_is = set(coll_list.get("DictList", {}))
        elif collection_class == "DataSpec":
            as_is = set(coll_list.get("SpecList", {}))

        dict_set = set(coll_list.get("DictList", {}))
        spec_set = set(coll_list.get("SpecList", {}))
        to_be_set = set(guid_list) if guid_list is not None else set()
        logger.debug(f"as_is: {as_is}")
        logger.debug(f"to_be_set: {to_be_set}")

        # Remove membership for collections that are in the as-is but not in the to-be
        to_remove = as_is - to_be_set
        logger.debug(f"to_remove: {to_remove}")
        if len(to_remove) > 0:
            remove_member_from_collections(egeria_client, to_remove, display_name, guid)

        # add membership for collections that are in the to-be but are not in the as-is
        to_add = to_be_set - as_is
        logger.debug(f"to_add: {to_add}")
        if len(to_add) > 0:
            add_member_to_collections(egeria_client, to_add, display_name, guid)
    else:
        add_member_to_collections(egeria_client, guid_list, display_name, guid)


#
# Product Manager Commands
#

@logger.catch
def process_collection_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    """
    Processes a digital product create or update object_action by extracting key attributes such as
    spec name, parent_guid, parent_relationship_type, parent_at_end_1, category

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """

    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        return None

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)

    guid = parsed_output.get('guid', None)

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    version = attributes['Version Identifier'].get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    output_set = make_format_set_name_from_type(object_type)
    if object_type in ["Root Collection", "Folder"]:
        obj = "Collection"
        if object_type == "Folder":
            qn_prefix = "Folder"
        elif object_type == "Root Collection":
            qn_prefix = "Root"

        qualified_name = egeria_client.__create_qualified_name__(qn_prefix, display_name, LOCAL_QUALIFIER,
                                                                 version_identifier=version)

    else:
        obj = object_type
    print(Markdown(parsed_output['display']))

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:

            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"`{object_action}`\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of `{command}` completed successfully! Proceeding to apply the changes.\n"))
                prop_body = set_element_prop_body(obj, qualified_name, attributes)

                body = set_update_body(obj, attributes)
                body['properties'] = set_element_prop_body(obj, qualified_name, attributes)

                egeria_client.update_collection(guid, body)
                if status:
                    egeria_client.update_collection_status(guid, status)

                logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                })
                return egeria_client.get_collection_by_guid(guid, element_type=obj,
                                                            output_format='MD', report_spec=output_set)


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"  Digital Product `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                else:
                    body = set_create_body(object_type, attributes)

                    # if this is a root or folder (maybe more in the future), then make sure that the classification is set.
                    body["initialClassifications"] = set_object_classifications(object_type, attributes,
                                                                                ["Folder", "Root Collection"])
                    body["properties"] = set_element_prop_body(obj, qualified_name, attributes)
                    parent_guid = body.get('parentGuid', None)
                    if parent_guid:
                        body['parentRelationshipTypeName'] = "CollectionMembership"
                        body['parentAtEnd1'] = True

                    guid = egeria_client.create_collection(body=body)
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                        })
                        msg = f"\nCreated Element `{display_name}` with GUID {guid}\n\n___"
                        # todo - add the source member asset to the product manager
                        # create_elem_from_template
                        # add this guid to the product collection
                        logger.success(msg)
                        return egeria_client.get_collection_by_guid(guid, obj, output_format='MD',
                                                                    report_spec=output_set)
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except PyegeriaException as e:
            logger.error(f"Pyegeria error performing {command}: {e}")
            print_basic_exception(e)
            return None
        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
    else:
        return None


@logger.catch
def process_digital_product_upsert_command(egeria_client: EgeriaTech, txt: str,
                                           directive: str = "display") -> Optional[str]:
    """
    Processes a digital product create or update object_action by extracting key attributes such as
    spec name, parent_guid, parent_relationship_type, parent_at_end_1, category

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """

    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    product_manager = attributes.get('Product Manager', {}).get('value', None)
    product_status = attributes.get('Product Status', {}).get('value', None)
    output_set = make_format_set_name_from_type(object_type)

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":

        try:
            prop_body = set_product_body(object_type, qualified_name, attributes)

            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"object_action\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                body = set_update_body(object_type, attributes)
                body['properties'] = prop_body
                # Todo: Update product manager later?

                egeria_client.update_digital_product(guid, body)
                # if product_status:
                #     egeria_client.update_digital_product_status(guid, product_status)

                logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                })
                return egeria_client.get_collection_by_guid(guid, element_type='Digital Product',
                                                            output_format='MD', report_spec=output_set)


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"  Digital Product `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                else:
                    body = set_create_body(object_type, attributes)
                    body["initialClassifications"] = set_object_classifications(object_type, attributes, [])

                    body["properties"] = prop_body

                    guid = egeria_client.create_digital_product(body_slimmer(body))
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                        })
                        # Todo: Add product manager link later? Agreements?

                        msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        return egeria_client.get_collection_by_guid(guid, element_type='Digital Product',
                                                                    output_format='MD', report_spec=output_set)
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None

    # @logger.catch
    # def process_digital_product_catalog_upsert_command(egeria_client: EgeriaTech, txt: str,
    #                                            directive: str = "display") -> Optional[str]:
    #     """
    #     Processes a digital product catalog create or update object_action by extracting key attributes such as
    #     spec name, parent_guid, parent_relationship_type, parent_at_end_1, category
    #
    #     :param txt: A string representing the input cell to be processed for
    #         extracting glossary-related attributes.
    #     :param directive: an optional string indicating the directive to be used - display, validate or execute
    #     :return: A string summarizing the outcome of the processing.
    #     """
    #
    #     command, object_type, object_action = extract_command_plus(txt)
    #     print(Markdown(f"# {command}\n"))
    #
    #     parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    #
    #     valid = parsed_output['valid']
    #     exists = parsed_output['exists']
    #
    #     qualified_name = parsed_output.get('qualified_name', None)
    #     guid = parsed_output.get('guid', None)
    #
    #     print(Markdown(parsed_output['display']))
    #
    #     logger.debug(json.dumps(parsed_output, indent=4))
    #
    #     attributes = parsed_output['attributes']
    #
    #     display_name = attributes['Display Name'].get('value', None)
    #
    #     output_set = make_format_set_name_from_type(object_type)
    #
    #     if directive == "display":
    #
    #         return None
    #     elif directive == "validate":
    #         if valid:
    #             print(Markdown(f"==> Validation of {command} completed successfully!\n"))
    #         else:
    #             msg = f"Validation failed for object_action `{command}`\n"
    #         return valid
    #
    #     elif directive == "process":
    #         try:
    #             prop_body = set_product_body(object_type, qualified_name, attributes)
    #
    #             if object_action == "Update":
    #                 if not exists:
    #                     msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
    #                            f"object_action\n")
    #                     logger.error(msg)
    #                     return update_a_command(txt, object_action, object_type, qualified_name, guid)
    #                 elif not valid:
    #                     return None
    #                 else:
    #                     print(Markdown(
    #                         f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
    #
    #                 body = set_update_body(object_type, attributes)
    #                 body['properties'] = prop_body
    #                 # Todo: Update product manager later?
    #
    #                 egeria_client.update_digital_product(guid, body)
    #                 # if product_status:
    #                 #     egeria_client.update_digital_product_status(guid, product_status)
    #
    #                 logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
    #                 update_element_dictionary(qualified_name, {
    #                     'guid': guid, 'display_name': display_name
    #                 })
    #                 return egeria_client.get_collection_by_guid(guid, element_type='Digital Product',
    #                                                             output_format='MD', report_spec=output_set)
    #
    #
    #             elif object_action == "Create":
    #                 if valid is False and exists:
    #                     msg = (
    #                         f"  Digital Product `{display_name}` already exists and result document updated changing "
    #                         f"`Create` to `Update` in processed output\n\n___")
    #                     logger.error(msg)
    #                     return update_a_command(txt, object_action, object_type, qualified_name, guid)
    #
    #                 else:
    #                     body = set_create_body(object_type, attributes)
    #                     body["initialClassifications"] = set_object_classifications(object_type, attributes, [])
    #
    #                     body["properties"] = prop_body
    #
    #                     guid = egeria_client.create_digital_product(body_slimmer(body))
    #                     if guid:
    #                         update_element_dictionary(qualified_name, {
    #                             'guid': guid, 'display_name': display_name
    #                         })
    #                         # Todo: Add product manager link later? Agreements?
    #
    #                         msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
    #                         logger.success(msg)
    #                         return egeria_client.get_collection_by_guid(guid, element_type='Digital Product',
    #                                                                     output_format='MD',
    #                                                                     report_spec=output_set)
    #                     else:
    #                         msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
    #                         logger.error(msg)
    #                         return None
    #
    #         except Exception as e:
    #             logger.error(f"Error performing {command}: {e}")
    #             return None
    #     else:
    #         return None


@logger.catch
def process_agreement_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    """
    Processes an agreement create or update object_action by extracting key attributes such as
    spec name, parent_guid, parent_relationship_type, parent_at_end_1, category

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        return None

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    output_set = make_format_set_name_from_type(object_type)

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            if object_type in ["Data Sharing Agreement"]:
                obj = "Agreement"
            else:
                obj = object_type

            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"object_action\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                prop_body = set_element_prop_body(obj, qualified_name, attributes)

                body = set_update_body(object_type, attributes)
                body['properties'] = set_element_prop_body(object_type, qualified_name, attributes)

                egeria_client.update_agreement(guid, body)
                # if status is not None and status !={}:
                #     egeria_client.update_agreement_status(guid, status)

                logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                })
                return egeria_client.get_collection_by_guid(guid, element_type='Data Specification',
                                                            output_format='MD', report_spec=output_set)


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"  Digital Product `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                else:
                    body = set_create_body(object_type, attributes)

                    # if this is a root or folder (maybe more in the future), then make sure that the classification is set.
                    body["initialClassifications"] = set_object_classifications(object_type, attributes,
                                                                                ["Data Sharing Agreement"])
                    body["properties"] = set_element_prop_body(obj, qualified_name, attributes)

                    guid = egeria_client.create_agreement(body=body)
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                        })
                        msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        return egeria_client.get_collection_by_guid(guid, obj, output_format='MD',
                                                                    report_spec=output_set)
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except PyegeriaException as e:
            logger.error(f"Pyegeria error performing {command}: {e}")
            print_exception_table(e)
            return None
        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
    else:
        return None


@logger.catch
def process_csv_element_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[
    str]:
    """
    Processes a create CSV element command by extracting key attributes and calling the pyegeria
    api that creates a csv element from template.

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        return None

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    output_set = make_format_set_name_from_type(object_type)

    file_name = attributes.get('File Name', {}).get('value', None)
    file_type = attributes.get('File Type', {}).get('value', None)
    file_path = attributes.get('File Path', {}).get('value', None)
    file_encoding = attributes.get('File Encoding', {}).get('value', 'UTF-8')
    file_extension = attributes.get('File Extension', {}).get('value', 'csv')
    file_system_name = attributes.get('File System Name', {}).get('value', None)
    version_identifier = attributes.get('Version Identifier', {}).get('value', None)
    description = attributes.get('Description', {}).get('value', None)

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:

            if object_action == "Create":
                if valid is False and exists:
                    msg = (f"  Element `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                else:
                    guid = egeria_client.get_create_csv_data_file_element_from_template(
                        file_name, file_type, file_path, version_identifier, file_encoding, file_extension,
                        file_system_name, description)

                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                        })
                        msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        output_md = (f"# Create CSV File\n\n## Display Name:\n\n {display_name}\n\n"
                                     f"## File Type:\n\n {file_type}\n\n## File Path:\n\n {file_path}\n\n"
                                     f"## File Encoding:\n\n {file_encoding}\n\n## File Extension:\n\n {file_extension}\n\n"
                                     f"## File System Name:\n\n {file_system_name}\n\n## Version Identifier:\n\n {version_identifier}\n\n"
                                     f"## Description:\n\n {description}\n\n"
                                     f"## Qualified Name\n\n {qualified_name}\n\n"
                                     f"## GUID:\n\n {guid}\n\n"
                                     )
                        return output_md
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None
            else:
                logger.error(f"Currently only the Create action is supported for this command: {command}")

        except PyegeriaException as e:
            logger.error(f"Pyegeria error performing {command}: {e}")
            print_exception_table(e)
            return None
        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
    else:
        logger.error(f"Invalid directive `{directive}`")
        return None


def process_link_agreement_item_command(egeria_client: EgeriaTech, txt: str,
                                        directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to add or remove an agreement item.

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    agreement = attributes.get('Agreement Name', {}).get('value', None)
    agreement_guid = attributes.get('Agreement Name', {}).get('guid', None)
    item = attributes.get('Item Name', {}).get('value', None)
    item_guid = attributes.get('Item Name', {}).get('guid', None)
    label = attributes.get('Link Label', {}).get('value', None)
    description = attributes.get('Description', {}).get('value', None)

    valid = parsed_output['valid']
    exists = agreement_guid is not None and item_guid is not None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":

        try:
            if object_action == "Detach":
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.detach_agreement_item(agreement_guid, item_guid, body)

                    logger.success(f"===> Detached agreement item `{item}` from agreement `{agreement}`\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)

            elif object_action == "Link":
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with Agreement `{agreement}` and item `{item}` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body(object_type, attributes)
                    item_props = {
                        "class": "AgreementItemProperties",
                        "agreementItemId": attributes.get("Agreement Item Id", {}).get("value", None),
                        "agreementItemTypeName": attributes.get("Agreement Item Type", {}).get("value", None),
                        "agreementStart": attributes.get("Agreement Start", {}).get("value", None),
                        "agreementEnd": attributes.get("Agreement End", {}).get("value", None),
                        "restrictions": attributes.get("Restrictions", {}).get("value", None),
                        "obligations": attributes.get("Obligations", {}).get("value", None),
                        "entitlements": attributes.get("Entitlements", {}).get("value", None),
                        "usageMeasurements": attributes.get("Usage Measurements", {}).get("value", None),
                        "effectiveFrom": attributes.get("Effective From", {}).get("value", None),
                        "effectiveTo": attributes.get("Effective To", {}).get("value", None)

                    }
                    body['properties'] = item_props
                    egeria_client.link_agreement_item(agreement_guid,
                                                      item_guid, body)
                    msg = f"==>Linked {object_type} `{agreement} to item {item}\n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out


        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None


def process_add_to_collection_command(egeria_client: EgeriaTech, txt: str,
                                      directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to add or remove a member to/from a collection..

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    element_guid = attributes.get('Element Id', {}).get('guid', None)
    collection_guid = attributes.get('Collection Id', {}).get('guid', None)
    membership_rationale = attributes.get('Membership Rationale', {}).get('value', None)
    expression = attributes.get('Expression', {}).get('value', None)
    confidence = attributes.get('Confidence', {}).get('value', None)
    membership_status = attributes.get('Membership Status', {}).get('value', None)
    user_defined_status = attributes.get('User Defined Status', {}).get('value', None)
    steward = attributes.get('Steward', {}).get('guid', None)
    steward_type_name = attributes.get('Steward Type Name', {}).get('value', None)
    steward_property_name = attributes.get('Steward Property Name', {}).get('value', None)
    source = attributes.get('Source', {}).get('value', None)
    notes = attributes.get('Notes', {}).get('value', None)
    glossary_term = attributes.get('Glossary Term', {}).get('value', None)
    journal_entry = attributes.get('Journal Entry', {}).get('value', None)

    valid = parsed_output['valid']
    # exists = agreement_guid is not None and  item_guid is not None
    exists = collection_guid is not None and element_guid is not None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        prop_body = {
            "class": "CollectionMembershipProperties",
            "membershipRationale": membership_rationale,
            "expression": expression,
            "membershipStatus": membership_status,
            "userDefinedStatus": user_defined_status,
            "confidence": confidence,
            "steward": steward,
            "stewardTypeName": steward_type_name,
            "stewardPropertyName": steward_property_name,
            "source": source,
            "notes": notes,
        }
        label = "Add Member"
        try:
            if object_action in ["Detach", "Unlink", "Remove"]:
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.remove_from_collection(collection_guid, element_guid, body)

                    logger.success(f"===> Detached element `{element_guid}` from collection `{collection_guid}`\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)

            elif object_action in ["Link", "Add", "Attach"]:
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `{label}` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body(object_type, attributes)

                    body['properties'] = prop_body
                    body = body_slimmer(body)
                    egeria_client.add_to_collection(collection_guid,
                                                    element_guid, body)
                    msg = f"==>Linked `{element_guid}` to collection `{collection_guid}` \n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out


        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None


def process_product_dependency_command(egeria_client: EgeriaTech, txt: str,
                                       directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to associate or break up a dependency between digital products..

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    digital_product1_guid = attributes.get('Digital Product 1', None)
    digital_product2_guid = attributes.get('Digital Product 2', None)
    label = attributes.get('Label', {}).get('value', None)
    description = attributes.get('Description', {}).get('value', None)
    effective_from = attributes.get('Effective From', {}).get('value', None)
    effective_to = attributes.get('Effective To', {}).get('value', None)

    valid = parsed_output['valid']
    exists = digital_product1_guid is not None and digital_product2_guid is not None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        prop_body = {
            "class": "DigitalProductDependencyProperties",
            "label": label,
            "description": description,
            "effectiveFrom": effective_from,
            "effectiveTo": effective_to
        }

        try:
            if object_action == "Detach":
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.detach_digital_product_dependency(digital_product1_guid, digital_product2_guid, body)

                    logger.success(
                        f"===> Detached dependency between products `{digital_product1_guid}` and `{digital_product2_guid}`\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)

            elif object_action == "Link":
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `{label}` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body(object_type, attributes)

                    body['properties'] = prop_body
                    egeria_client.link_digital_product_dependency(digital_product1_guid,
                                                                  digital_product2_guid, body)
                    msg = f"==>Linked dependency from digital product `{digital_product1_guid}` to product `{digital_product2_guid}` \n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out


        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None


def process_attach_collection_command(egeria_client: EgeriaTech, txt: str,
                                      directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to attach a collection to a resources.

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    collection_guid = attributes.get('Collection Id', {}).get('guid', None)
    resource_guid = attributes.get('Resource Id', {}).get('guid', None)
    resource_use = attributes.get('Resource Use', {}).get('value', None)
    resource_description = attributes.get('Resource Description', {}).get('value', None)
    resource_properties = attributes.get('Resource Properties', {}).get('value', None)
    effective_from = attributes.get('Effective From', {}).get('value', None)
    effective_to = attributes.get('Effective To', {}).get('value', None)

    valid = parsed_output['valid']
    exists = collection_guid is not None and resource_guid is not None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        prop_body = {
            "class": "ResourceListProperties",
            "resourceUse": resource_use,
            "resourceDescription": resource_description,
            "resourceProperties": resource_properties,
            "effectiveFrom": effective_from,
            "effectiveTo": effective_to
        }

        try:
            if object_action == "Detach":
                if not exists:
                    msg = (f" Link `Attach Resource to Collection` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.detach_collection(resource_guid, collection_guid, body)

                    logger.success(
                        f"===> Detached linkage between resource `{resource_guid}` and collection`{collection_guid}`\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)

            elif object_action == "Link":
                if valid is False and exists:
                    msg = (
                        f"-->  Link called `Attach Resource to Collection` already exists and result document updated changing "
                        f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `Attach Resource to Collection` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body(object_type, attributes)

                    body['properties'] = prop_body
                    egeria_client.attach_collection(resource_guid,
                                                    collection_guid, body)
                    msg = f"==>Attached collection `{collection_guid}` to resource `{resource_guid}` \n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out


        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None


def process_attach_subscriber_command(egeria_client: EgeriaTech, txt: str,
                                      directive: str = "display") -> Optional[str]:
    """
    Processes a link or unlink command to attach a subscriber to a subscription.

    :param txt: A string representing the input cell to be processed for
        extracting blueprint-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)
    if parsed_output is None:
        logger.error(f"Input error in command `{txt}`")
        return None

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    subscriber_guid = attributes.get('Subscriber Id', {}).get('guid', None)
    subscription_guid = attributes.get('Subscription', {}).get('guid', None)

    valid = parsed_output['valid']
    exists = subscriber_guid is not None and subscription_guid is not None
    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        prop_body = {
            "class": "DigitalSubscriberProperties",
            "subscriberId": attributes.get('Subscriber Id', {}).get('value', None),
            "effectiveFrom": attributes.get('Effective From', {}).get('value', None),
            "effectiveTo": attributes.get('Effective To', {}).get('value', None),
        }

        try:
            if object_action == "Detach":
                if not exists:
                    msg = (
                        f" Link `Attach Subscriber to Subscription` does not exist! Updating result document with Link "
                        f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.detach_subscriber(subscriber_guid, subscription_guid, body)

                    logger.success(
                        f"===> Detached linkage between subscriber `{subscriber_guid}` and subscription`{subscription_guid}`\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)

            elif object_action == "Link":
                if valid is False and exists:
                    msg = (
                        f"-->  Link called `Attach Subscriber to Subscription` already exists and result document updated changing "
                        f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Subscription link `{subscriber_guid}` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body(object_type, attributes)

                    body['properties'] = prop_body
                    egeria_client.link_subscriber(subscriber_guid,
                                                  subscription_guid, body)
                    msg = f"==>Attached subscriber `{subscriber_guid}` to subscription `{subscriber_guid}` \n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out


        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


This file contains a set of test routines to test the server operations services of the Egeria python client.
The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests. A set of platform, server and user variables are
created local to the TestPlatform class to hold the set of values to be used for testing.  It should, however, be noted that the tests are currently
order sensitive - in other words if you delete all the servers the subsequent tests that expect the servers to be
available may fail..

"""

import json

import pytest
from rich import print, print_json

from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    print_exception_response,
)
from pyegeria.server_operations import ServerOps

disable_ssl_warnings = True


class TestServerOperations:
    good_platform1_url = "https://127.0.0.1:9443"
    good_platform2_url = "https://localhost:9444"
    bad_platform1_url = "https://localhost:9443"

    # good_platform1_url = "https://127.0.0.1:30080"
    # good_platform2_url = "https://127.0.0.1:30081"
    # bad_platform1_url = "https://localhost:9443"

    good_user_1 = "garygeeke"
    good_user_2 = "erinoverview"
    bad_user_1 = "eviledna"
    bad_user_2 = ""

    good_server_1 = "active-metadata-store"
    good_server_2 = "qs-integration-daemon"
    good_server_3 = "exchangeDL01"
    good_server_4 = "engine-host"
    bad_server_1 = "coco"
    bad_server_2 = ""

    def test_get_active_configuration(self, server: str = good_server_1):
        server_name = server
        try:
            s_client = ServerOps(server_name, self.good_platform1_url, self.good_user_1)
            response = s_client.get_active_configuration(server)
            print(f"\n\n\tThe active configuration of {server} is \n{response}")
            assert True

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "404", "Invalid parameters"

    def test_add_archive_files(self):
        # Todo - the base function doesn't seem to validate the file or to actually load? Check
        try:
            # server = self.good_server_1
            server = "cocoMDS2"
            p_client = ServerOps(server, self.good_platform1_url, self.good_user_1)
            p_client.add_archive_file("content-packs/CoreContentPack.omarchive", server)
            assert True, "Should have raised an exception"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "404", "Invalid parameters"

    @pytest.mark.skip(reason="Need to find a good archive connection body")
    def test_add_archive(self):
        # Todo - the base function doesn't seem to validate the file or to actually load? Check
        try:
            server = self.good_server_1
            p_client = ServerOps(server, self.good_platform2_url, self.good_user_1)
            p_client.add_archive("./cocoGovernanceEngineDefinition.json", server)

            assert True, "Should have raised an exception"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "404", "Invalid parameters"

    def test_get_active_server_status(self, server: str = good_server_1):
        try:
            p_client = ServerOps(server, self.good_platform1_url, self.good_user_1)
            response = p_client.get_active_server_status(server)
            print(response)
            assert response.get("relatedHTTPCode") == 200, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "404", "Invalid parameters"

    def test_get_active_service_list_for_server(self):
        try:
            server = self.good_server_1
            p_client = ServerOps(server, self.good_platform1_url, self.good_user_1)
            response = p_client.get_active_service_list_for_server(server)
            print(f"\n\n\tActive Service list for server {server} is {response}")
            assert True, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

    def test_get_governance_engine_summaries(self, server: str = good_server_4):
        try:
            server_name = server
            s_client = ServerOps(server_name, self.good_platform1_url, self.good_user_1)
            response = s_client.get_governance_engine_summaries(server)
            print(
                f"\n\n\tGovernance Engine Summary for server {server} is {json.dumps(response, indent=4)}"
            )

            assert True, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

    #
    #   Integration Daemon Ops
    #

    def test_get_integration_daemon_status(self, server: str = good_server_2):
        try:
            server_name = server
            s_client = ServerOps(server_name, self.good_platform1_url, self.good_user_1)
            response = s_client.get_integration_daemon_status(server)

            if type(response) is dict:
                # print(f"\n\n\tIntegration Daemon Status for server {server} is {json.dumps(response, indent=4)}")
                print(f"\n\n\tIntegration Daemon Status for server {server} is:")
                print_json(json.dumps(response))
            else:
                print(f"\n\n\tIntegration Daemon Status response was: {response}")
            assert True, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

    def test_get_connector_config(self, server: str = good_server_2):
        try:
            server_name = server
            connector = "JDBCDatabaseCataloguer"
            s_client = ServerOps(server_name, self.good_platform1_url, self.good_user_1)
            response = s_client.get_connector_config(connector, server)
            print(
                f"\n\n\tConnector configuration for connector {connector} is \n{json.dumps(response, indent=4)}"
            )

            assert True, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

    # todo - review with Mandy?
    def test_restart_integration_connector(self, server: str = good_server_2):
        try:
            server_name = "integration-daemon"
            # connector = "FilesMonitor"
            # connector = "DataFilesMonitorIntegrationConnector"

            connector = None
            s_client = ServerOps(server_name, self.good_platform1_url, self.good_user_1)
            # response = s_client.restart_integration_connector(connector,server)
            s_client.restart_integration_connector(connector, server_name)

            assert True, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

    def test_refresh_integration_connectors(self, server: str = good_server_3):
        try:
            server_name = "integration-daemon"
            connector = "FilesMonitor"
            s_client = ServerOps(server_name, self.good_platform1_url, self.good_user_1)

            s_client.refresh_integration_connectors("all", server_name)

            assert True, "Invalid URL or server"

        except (InvalidParameterException, PropertyServerException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"



"""
This is an ongoing experiment in parsing and playing with Freddie docs
"""
import os
import sys
from datetime import datetime

from loguru import logger
from pydantic import ValidationError

log_format = "{time} | {level} | {function} | {line} | {message} | {extra}"
logger.remove()
logger.add(sys.stderr, level="INFO", format=log_format, colorize=True)
logger.add("debug_log.log", rotation="1 day", retention="1 week", compression="zip", level="TRACE", format=log_format,
           colorize=True)
import click
from rich import print
from rich.console import Console


from pyegeria._exceptions_new import PyegeriaException
from md_processing.dr_egeria import process_md_file

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get("EGERIA_VIEW_SERVER_URL", "https://localhost:9443")
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get("EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443")
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_WIDTH = os.environ.get("EGERIA_WIDTH", 220)
EGERIA_JUPYTER = os.environ.get("EGERIA_JUPYTER", False)
EGERIA_HOME_GLOSSARY_GUID = os.environ.get("EGERIA_HOME_GLOSSARY_GUID", None)
EGERIA_GLOSSARY_PATH = os.environ.get("EGERIA_GLOSSARY_PATH", None)
EGERIA_ROOT_PATH = os.environ.get("EGERIA_ROOT_PATH", "../..")
EGERIA_INBOX_PATH = os.environ.get("EGERIA_INBOX_PATH", "md_processing/dr_egeria_inbox")
EGERIA_OUTBOX_PATH = os.environ.get("EGERIA_OUTBOX_PATH", "md_processing/dr_egeria_outbox")

@click.command("process_markdown_file", help="Process a markdown file and return the output as a string.")
@click.option("--input-file", help="Markdown file to process.", default="dr_egeria_intro_part1.md", required=True,
              prompt=False)
@click.option("--output-folder", help="Output folder.", default="", required=False)
@click.option("--directive", default="process", help="How to process the file",
              type=click.Choice(["display", "validate", "process"], case_sensitive=False), prompt=False, )
@click.option("--server", default=EGERIA_VIEW_SERVER, help="Egeria view server to use.")
@click.option("--url", default=EGERIA_VIEW_SERVER_URL, help="URL of Egeria platform to connect to")
@click.option("--userid", default=EGERIA_USER, help="Egeria user")
@click.option("--user_pass", default=EGERIA_USER_PASSWORD, help="Egeria user password")
@logger.catch
def process_markdown_file(input_file: str, output_folder:str, directive: str, server: str, url: str, userid: str,
                          user_pass: str ) -> None:
    """
    Process a markdown file by parsing and executing Dr. Egeria md_commands. Write output to a new file.
    """
    try:
        process_md_file(input_file, output_folder, directive, server, url, userid, user_pass)
        logger.info(f"Called process_markdown_file with input file {input_file}")
    except PyegeriaException as e:
        logger.error(f"Error processing markdown file {input_file}: {e}")
    except Exception as e:
        logger.error(f"Unexpected error processing markdown file {input_file}: {e}")


# def main():
#     parser = argparse.ArgumentParser()
#     parser.add_argument("--server", help="Name of the server to display status for")
#     parser.add_argument("--url", help="URL Platform to connect to")
#     parser.add_argument("--userid", help="User Id")
#     parser.add_argument("--password", help="User Password")
#     parser.add_argument("--time_out", help="Time Out")
#
#     args = parser.parse_args()
#
#     server = args.server if args.server is not None else EGERIA_VIEW_SERVER
#     url = args.url if args.url is not None else EGERIA_PLATFORM_URL
#     userid = args.userid if args.userid is not None else EGERIA_USER
#     user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
#     time_out = args.time_out if args.time_out is not None else 60
#     try:
#         input_file = Prompt.ask("Markdown File name to process:", default="")
#         directive = Prompt.ask("Processing Directive:", choices=[ "display", "validate", "process"],
#         default="validate")
#
#         process_markdown_file(input_file, directive, server, url, userid, user_pass)
#     except KeyboardInterrupt:
#         pass
#
#
# if __name__ == "__main__":
#     main()

if __name__ == "__main__":
    process_markdown_file()


"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

This is the pyegeria client package. The purpose of the package is to provide
easy access to Egeria (https://egeria-project.org). The package is currently in
development.

The first capabilities are around Egeria's platform services used to start and stop
the server platform and servers.

"""

import orig_globals
